/*
 * gen-gammachart.vala - Gamma chart generator
 *
 * Copyright (C) 2011 kakurasan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * compile:
 *  valac --pkg posix --pkg gdk-pixbuf-2.0 gen-gammachart.vala -o gen-gammachart
 */

using Posix;
using Gdk;

namespace GenGammaChart
{
  enum ColorValue
  {
    V000,
    V025,
    V050,
    V075,
    V100
  }
  /**
   * XPM data
   */
  class Xpm
  {
    public static const int size = 8;
    public static const int size2x = size * 2;
    protected string[] _data = {};
    public string[] data
    {
      get
      {
        return this._data;
      }
    }
  }
  /**
   * XPM data (single color)
   */
  class SingleColorXpm : Xpm
  {
    public
    SingleColorXpm (string red, string green, string blue)
    {
      this._data += @"$(Xpm.size2x) $(Xpm.size2x) 1 1";
      this._data += @" c #$(red)$(green)$(blue)";
      for (var i = 0; i < Xpm.size2x; i++)
      {
        var sb = new GLib.StringBuilder ();
        for (var j = 0; j < Xpm.size2x; j++)
          sb.append (" ");
        this._data += (sb.str);
      }
    }
  }
  /**
   * XPM data (alternate)
   */
  class AlternateXpm : Xpm
  {
    public
    AlternateXpm (string red1, string green1, string blue1, string red2, string green2, string blue2)
    {
      this._data += @"$(Xpm.size2x) $(Xpm.size2x) 2 1";
      this._data += @" c #$(red1)$(green1)$(blue1)";
      this._data += @".c #$(red2)$(green2)$(blue2)";
      for (var i = 0; i < Xpm.size; i++)
      {
        var sb = new GLib.StringBuilder ();
        for (var j = 0; j < Xpm.size2x; j++)
          sb.append (". ");
        this._data += (sb.str);
        sb = new GLib.StringBuilder ();
        for (var j = 0; j < Xpm.size2x; j++)
          sb.append (" .");
        this._data += (sb.str);
      }
    }
  }
  int
  main (string[] args)
  {
    string outdir;
    double[] gammalist = {2.2, 2.0, 1.8, 1.0};
    const int xpm_side = Xpm.size * 2;
    const int image_side = Xpm.size * 24;
    if (args.length < 2)
    {
      print (@"USAGE: $(args[0]) [OUTDIR]\n");
      return EXIT_SUCCESS;
    }
    outdir = args[1];
    if (! GLib.FileUtils.test (outdir, GLib.FileTest.IS_DIR))
    {
      if (GLib.DirUtils.create (outdir, 00755) != 0)
      {
        printerr (@"Error: Could not create directory \"$(outdir)\"\n");
        return EXIT_FAILURE;
      }
    }
    foreach (var gamma in gammalist)
    {
      var pb_out = new Gdk.Pixbuf (Gdk.Colorspace.RGB, false, 8, image_side, image_side);
      var outfile = GLib.Path.build_filename (outdir, "%0.1f.png".printf (gamma));
      Gdk.Pixbuf[] arr_pb_alternate = {};
      Gdk.Pixbuf[] arr_pb_single = {};
      string[] values = {};
      values += "%02x".printf ((uint) Posix.round (Posix.pow (  0 / 255.0, 1.0 / gamma) * 255));
      values += "%02x".printf ((uint) Posix.round (Posix.pow ( 63 / 255.0, 1.0 / gamma) * 255));
      values += "%02x".printf ((uint) Posix.round (Posix.pow (127 / 255.0, 1.0 / gamma) * 255));
      values += "%02x".printf ((uint) Posix.round (Posix.pow (191 / 255.0, 1.0 / gamma) * 255));
      values += "%02x".printf ((uint) Posix.round (Posix.pow (255 / 255.0, 1.0 / gamma) * 255));
      // all
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V050], values[ColorValue.V050], values[ColorValue.V050]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V025], values[ColorValue.V025], values[ColorValue.V025]).data);
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V100], values[ColorValue.V100], values[ColorValue.V100]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V050], values[ColorValue.V050], values[ColorValue.V050]).data);
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V050], values[ColorValue.V050], values[ColorValue.V050], values[ColorValue.V100], values[ColorValue.V100], values[ColorValue.V100]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V075], values[ColorValue.V075], values[ColorValue.V075]).data);
      // red
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V050], values[ColorValue.V000], values[ColorValue.V000]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V025], values[ColorValue.V000], values[ColorValue.V000]).data);
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V100], values[ColorValue.V000], values[ColorValue.V000]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V050], values[ColorValue.V000], values[ColorValue.V000]).data);
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V050], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V100], values[ColorValue.V000], values[ColorValue.V000]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V075], values[ColorValue.V000], values[ColorValue.V000]).data);
      // green
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V050], values[ColorValue.V000]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V000], values[ColorValue.V025], values[ColorValue.V000]).data);
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V100], values[ColorValue.V000]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V000], values[ColorValue.V050], values[ColorValue.V000]).data);
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V050], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V100], values[ColorValue.V000]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V000], values[ColorValue.V075], values[ColorValue.V000]).data);
      // blue
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V050]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V025]).data);
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V100]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V050]).data);
      arr_pb_alternate += new Gdk.Pixbuf.from_xpm_data (new AlternateXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V050], values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V100]).data);
      arr_pb_single += new Gdk.Pixbuf.from_xpm_data (new SingleColorXpm (values[ColorValue.V000], values[ColorValue.V000], values[ColorValue.V075]).data);
      // build
      for (var i = 0; i < 12; i++)
      {
        for (var j = 0; j < 3; j++)
        {
          var k = j + (i / 3) * 3;
          var l = i % 3;
          if (j % 2 == 0)
          {
            arr_pb_alternate[i].copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4), xpm_side * k);
            arr_pb_alternate[i].copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 2, xpm_side * k);
            arr_pb_single[i].copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 1, xpm_side * k);
            arr_pb_single[i].copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 3, xpm_side * k);
          }
          else
          {
            arr_pb_alternate[i].copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side, xpm_side * k);
            arr_pb_alternate[i].copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 3, xpm_side * k);
            arr_pb_single[i].copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 2, xpm_side * k);
            arr_pb_single[i].copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4), xpm_side * k);
          }
        }
      }
      // save
      try
      {
        pb_out.save (outfile, "png");
        print (@"Wrote \"$(outfile)\".\n");
      }
      catch (GLib.Error e)
      {
        printerr (@"Error: Could not write \"$(outfile)\": $(e.message)\n");
        return EXIT_FAILURE;
      }
    }
    return EXIT_SUCCESS;
  }
}
