# -*- mode: python; python-indent: 2 -*-
#
# xbrightness-rgbgamma - X11 brightness / gamma adjustment tool
#
# Copyright (C) 2011 kakurasan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# THANKS:
#  xbrightness by Asher Blum and Jean-Daniel Pauget

from __future__ import print_function

from xbrightness_rgbgamma.AppInfo import AppInfo
import gettext
import sys
import os


def main (cmd):
  """
  Main
  """
  # NLS
  gettext.install (AppInfo.pkgname, os.path.join (AppInfo.prefix, 'share', 'locale'))

  # load/check Libxxf86vm
  try:
    import xbrightness_rgbgamma.Libxxf86vm as Libxxf86vm
  except ImportError as e:
    return _('Error: ') + _('Could not load libXxf86vm') + ': {0}'.format (e)

  # help/version
  print_help = print_version = False
  for a in sys.argv[1:]:
    if a == '-h' or a == '--help':
      print_help = True
    elif a == '-v' or a == '--version':
      print_version = True
  if len (sys.argv) < 2 or print_help:
    print (_('''USAGE:
 {0} [BRIGHTNESS]
 {0} [BRIGHTNESS] [GAMMA]
 {0} [BRIGHTNESS] [GAMMA_RED] [GAMMA_GREEN] [GAMMA_BLUE]
 {0} [GAMMA_RED] [GAMMA_GREEN] [GAMMA_BLUE]

BRIGHTNESS: 0 - 65535
GAMMA,GAMMA_RED,GAMMA_GREEN,GAMMA_BLUE: 0.000..1 - 10''').format (cmd))
    return None
  if print_version:
    print (_('{0} version {1}\nCopyright {2}').format (AppInfo.name, AppInfo.version, AppInfo.copyright))
    return None

  if len (sys.argv) == 4:
    # gamma (red + green + blue)
    gamma_red = sys.argv[1]
    gamma_green = sys.argv[2]
    gamma_blue = sys.argv[3]
  else:
    brightness = sys.argv[1]
    if len (sys.argv) == 3:
      # brightness + gamma (all)
      gamma_red = gamma_green = gamma_blue = sys.argv[2]
    elif len (sys.argv) == 5:
      # brightness + gamma (red + green + blue)
      gamma_red = sys.argv[2]
      gamma_green = sys.argv[3]
      gamma_blue = sys.argv[4]

  if len (sys.argv) != 4:
    # check invalid brightness
    try:
      brightness = int (brightness)
    except ValueError:
      return _('Error: ') + _('Invalid value "{0}"').format (brightness)
    # check range
    if brightness < 0 or brightness > 65535:
      return _('Error: ') + _('Invalid brightness (range: 0 - 65535)')
  if len (sys.argv) > 2:
    # check invalid gamma
    try:
      gamma_red = float (gamma_red)
    except ValueError:
      return _('Error: ') + _('Invalid value "{0}"').format (gamma_red)
    try:
      gamma_green = float (gamma_green)
    except ValueError:
      return _('Error: ') + _('Invalid value "{0}"').format (gamma_green)
    try:
      gamma_blue = float (gamma_blue)
    except ValueError:
      return _('Error: ') + _('Invalid value "{0}"').format (gamma_blue)
    # check range
    if gamma_red <= 0 or gamma_green <= 0 or gamma_blue <= 0:
        return _('Error: ') + _('Gamma must be greater than 0')
    if gamma_red > 10 or gamma_green > 10 or gamma_blue > 10:
        return _('Error: ') + _('Gamma must be less or equal to 10')

  # start
  try:
    lib = Libxxf86vm.Libxxf86vm ()
  except Libxxf86vm.XOpenDisplayError:
    try:
      return _('Error: ') + _('Could not open display: ') + '"{0}"'.format (os.environ['DISPLAY'])
    except KeyError:
      return _('Error: ') + _('Could not open display: ') + _('(undefined)')
  except Libxxf86vm.GetGammaRampSizeError:
    return _('Error: ') + _('Could not get size of gamma ramp.')
  except Libxxf86vm.GetGammaRampError:
    return _('Error: ') + _('Could not get gamma ramp.')
  try:
    if len (sys.argv) == 2:
      lib.set_brightness (brightness)
    elif len (sys.argv) == 4:
      lib.set_gamma (gamma_red, gamma_green, gamma_blue)
    else:  # 3 or 5
      lib.set_gamma_and_brightness (brightness, gamma_red, gamma_green, gamma_blue)
  except Libxxf86vm.SetGammaRampError:
    return _('Error: ') + _('Could not set gamma ramp.')

  return None
