# -*- mode: python; python-indent: 2 -*-
#
# xbrightness-rgbgamma - X11 brightness / gamma adjustment tool
#
# Copyright (C) 2011 kakurasan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# THANKS:
#  xbrightness by Asher Blum and Jean-Daniel Pauget

import ctypes
import math
import sys


class GetGammaRampSizeError (Exception):
  pass

class GetGammaRampError (Exception):
  pass

class SetGammaRampError (Exception):
  pass

class XOpenDisplayError (Exception):
  pass

class Libxxf86vm:
  __ramp_size = ctypes.c_int ()
  try:
    __lib = ctypes.CDLL ('libXxf86vm.so.1')
  except OSError as e:
    raise ImportError (e)
  def __init__ (self):
    # XOpenDisplay(): Display *
    self.__display = self.__lib.XOpenDisplay (None)  # envar "DISPLAY"
    if not self.__display:
      raise XOpenDisplayError ()
    # XDefaultScreen: Display *
    self.__screen = self.__lib.XDefaultScreen (self.__display)
    # XF86VidModeGetGammaRampSize(): Display *, int, int *
    ret = self.__lib.XF86VidModeGetGammaRampSize (self.__display, self.__screen, ctypes.byref (self.__ramp_size))
    if not ret:
      raise GetGammaRampSizeError ()
    Ramp = ctypes.c_ushort * self.__ramp_size.value  # type
    (self.__ramp_red, self.__ramp_green, self.__ramp_blue) = (Ramp (), Ramp (), Ramp ())
    # XF86VidModeGetGammaRamp(): Display *, int, int,
    #                            unsigned short *, unsigned short *, unsigned short *
    ret = self.__lib.XF86VidModeGetGammaRamp (self.__display, self.__screen, self.__ramp_size, self.__ramp_red, self.__ramp_green, self.__ramp_blue)
    if not ret:
      raise GetGammaRampError ()
    # max output -> brightness
    self.__brightness = self.__ramp_red[self.__ramp_size.value - 1]
    # calculate current gamma
    self.__gamma_red = float ('{0:.2f}'.format (1.0 / math.log (self.__ramp_red[int (self.__ramp_size.value / 2)] / float (self.__brightness), (self.__ramp_size.value / 2) / float (self.__ramp_size.value - 1))))
    self.__gamma_green = float ('{0:.2f}'.format (1.0 / math.log (self.__ramp_green[int (self.__ramp_size.value / 2)] / float (self.__brightness), (self.__ramp_size.value / 2) / float (self.__ramp_size.value - 1))))
    self.__gamma_blue = float ('{0:.2f}'.format (1.0 / math.log (self.__ramp_blue[int (self.__ramp_size.value / 2)] / float (self.__brightness), (self.__ramp_size.value / 2) / float (self.__ramp_size.value - 1))))
  def __del__ (self):
    if self.__display:
      # XCloseDisplay(): Display *
      self.__lib.XCloseDisplay (self.__display)
  def set_gamma (self, gamma_red, gamma_green, gamma_blue):
    self.set_gamma_and_brightness (self.__brightness, gamma_red, gamma_green, gamma_blue)
  def set_brightness (self, brightness):
    self.set_gamma_and_brightness (brightness, self.__gamma_red, self.__gamma_green, self.__gamma_blue)
  def set_gamma_and_brightness (self, brightness, gamma_red, gamma_green, gamma_blue):
    for i in range (self.__ramp_size.value):
      p = i / float (self.__ramp_size.value - 1)
      self.__ramp_red[i] = ctypes.c_ushort (int (brightness * pow (p, 1 / gamma_red)))
      self.__ramp_green[i] = ctypes.c_ushort (int (brightness * pow (p, 1 / gamma_green)))
      self.__ramp_blue[i] = ctypes.c_ushort (int (brightness * pow (p, 1 / gamma_blue)))
    # XF86VidModeSetGammaRamp(): Display *, int, int,
    #                            unsigned short *, unsigned short *, unsigned short *
    ret = self.__lib.XF86VidModeSetGammaRamp (self.__display, self.__screen, self.__ramp_size, self.__ramp_red, self.__ramp_green, self.__ramp_blue)
    if not ret:
      raise SetGammaRampError ()
    self.__brightness = brightness
    (self.__gamma_red, self.__gamma_green, self.__gamma_blue) = (gamma_red, gamma_green, gamma_blue)
  @property
  def brightness (self): return self.__brightness
  @property
  def gamma_red (self): return self.__gamma_red
  @property
  def gamma_green (self): return self.__gamma_green
  @property
  def gamma_blue (self): return self.__gamma_blue
  @property
  def ramp_red (self): return self.__ramp_red
  @property
  def ramp_green (self): return self.__ramp_green
  @property
  def ramp_blue (self): return self.__ramp_blue
  @property
  def ramp_size (self): return self.__ramp_size.value
